@@ -17,13 +17,17 @@
 -- Portability :  non-portable
 --
 -- This module provides a shim around 'coerce' that defaults to 'unsafeCoerce'
--- on GHC < 7.8
+-- on GHC < 7.8. It also exposes a type-restricted version of '(#.)' that
+-- works around a bizarre GHC 7.10–specific bug.
 -----------------------------------------------------------------------------
 module Control.Lens.Internal.Coerce
   ( coerce
   , coerce'
+  , (#..)
   ) where
 
+import Data.Profunctor.Unsafe
+
 #ifdef USE_COERCE
 
 import Data.Coerce
@@ -32,6 +36,10 @@ coerce' :: forall a b. Coercible a b => b -> a
 coerce' = coerce (id :: a -> a)
 {-# INLINE coerce' #-}
 
+(#..) :: (Profunctor p, Coercible c b) => (b -> c) -> p a b -> p a c
+(#..) = (#.)
+{-# INLINE (#..) #-}
+
 #else
 
 import Unsafe.Coerce
@@ -41,4 +49,8 @@ coerce  = unsafeCoerce
 coerce' = unsafeCoerce
 {-# INLINE coerce #-}
 {-# INLINE coerce' #-}
+
+(#..) :: Profunctor p => (b -> c) -> p a b -> p a c
+(#..) = (#.)
+{-# INLINE (#..) #-}
 #endif
