@@ -2,6 +2,7 @@
 {-# LANGUAGE Rank2Types #-}
 {-# LANGUAGE FunctionalDependencies #-}
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TypeFamilies #-}
 
 #if __GLASGOW_HASKELL__ >= 710
 {-# LANGUAGE PatternSynonyms #-}
@@ -9,6 +10,10 @@
 {-# LANGUAGE Trustworthy #-}
 #endif
 
+#if __GLASGOW_HASKELL__ >= 800
+{-# LANGUAGE TypeInType #-}
+#endif
+
 #ifndef MIN_VERSION_bytestring
 #define MIN_VERSION_bytestring(x,y,z) 1
 #endif
@@ -20,7 +25,7 @@
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
--- Portability :  Rank2Types
+-- Portability :  Rank2Types, TypeFamilies, FunctionalDependencies
 --
 ----------------------------------------------------------------------------
 module Control.Lens.Iso
@@ -37,6 +42,8 @@ module Control.Lens.Iso
   -- * Working with isomorphisms
   , au
   , auf
+  , xplat
+  , xplatf
   , under
   , mapping
   -- ** Common Isomorphisms
@@ -99,16 +106,21 @@ import Control.Lens.Review
 import Control.Lens.Type
 import Control.Monad.State.Lazy as Lazy
 import Control.Monad.State.Strict as Strict
-import Control.Monad.Writer.Lazy as Lazy
-import Control.Monad.Writer.Strict as Strict
-import Control.Monad.RWS.Lazy as Lazy
-import Control.Monad.RWS.Strict as Strict
+import Control.Monad.Writer.Lazy as Lazy hiding (Product, Sum)
+import Control.Monad.Writer.Strict as Strict hiding (Product, Sum)
+import Control.Monad.RWS.Lazy as Lazy hiding (Product, Sum)
+import Control.Monad.RWS.Strict as Strict hiding (Product, Sum)
 import Control.Monad.ST.Lazy as Lazy
 import Control.Monad.ST as Strict
+
 import Data.Bifunctor
+import Data.Bifunctor.Biff
+import Data.Bifunctor.Flip
+import Data.Bifunctor.Product
+import Data.Bifunctor.Sum
+import Data.Bifunctor.Tannen
 import Data.ByteString as StrictB hiding (reverse)
 import Data.ByteString.Lazy as LazyB hiding (reverse)
-
 import Data.Functor.Identity
 import Data.Text as StrictT hiding (reverse)
 import Data.Text.Lazy as LazyT hiding (reverse)
@@ -117,6 +129,10 @@ import Data.Maybe
 import Data.Profunctor
 import Data.Profunctor.Unsafe
 
+#if !(MIN_VERSION_base(4,8,0))
+import Data.Functor
+#endif
+
 #if __GLASGOW_HASKELL__ >= 708
 import Data.Coerce (Coercible)
 #if __GLASGOW_HASKELL__ < 710
@@ -128,6 +144,10 @@ import Data.Type.Coercion
 import qualified GHC.Exts as Exts
 #endif
 
+#if __GLASGOW_HASKELL__ >= 800
+import GHC.Exts (TYPE)
+#endif
+
 #ifdef HLINT
 {-# ANN module "HLint: ignore Use on" #-}
 #endif
@@ -172,12 +192,17 @@ iso sa bt = dimap sa (fmap bt)
 -- 'from' ('from' l) ≡ l
 -- @
 from :: AnIso s t a b -> Iso b a t s
-from l = withIso l $ \ sa bt -> iso bt sa
+from l = withIso l $ flip iso
 {-# INLINE from #-}
 
 -- | Extract the two functions, one from @s -> a@ and
 -- one from @b -> t@ that characterize an 'Iso'.
+#if __GLASGOW_HASKELL__ >= 800
+withIso :: forall s t a b rep (r :: TYPE rep).
+             AnIso s t a b -> ((s -> a) -> (b -> t) -> r) -> r
+#else
 withIso :: AnIso s t a b -> ((s -> a) -> (b -> t) -> r) -> r
+#endif
 withIso ai k = case ai (Exchange id Identity) of
   Exchange sa bt -> k sa (runIdentity #. bt)
 {-# INLINE withIso #-}
@@ -208,6 +233,10 @@ cloneIso k = withIso k iso
 -- @
 -- au :: AnIso s t a b -> ((b -> t) -> e -> s) -> e -> a
 -- @
+--
+-- @
+-- au = xplat . from
+-- @
 au :: Functor f => AnIso s t a b -> ((b -> t) -> f s) -> f a
 au k = withIso k $ \ sa bt f -> fmap sa (f bt)
 {-# INLINE au #-}
@@ -218,21 +247,41 @@ au k = withIso k $ \ sa bt f -> fmap sa (f bt)
 --
 -- For a version you pass the name of the @newtype@ constructor to, see 'Control.Lens.Wrapped.alaf'.
 --
--- >>> auf (_Unwrapping Sum) (foldMapOf both) Prelude.length ("hello","world")
+-- >>> auf (_Wrapping Sum) (foldMapOf both) Prelude.length ("hello","world")
 -- 10
 --
 -- Mnemonically, the German /auf/ plays a similar role to /à la/, and the combinator
 -- is 'au' with an extra function argument:
 --
 -- @
--- 'auf' :: 'Iso' s t a b -> ((r ->  a) -> e -> b) -> (r -> s) -> e -> t
+-- 'auf' :: 'Iso' s t a b -> ((r -> t) -> e -> s) -> (r -> b) -> e -> a
 -- @
 --
 -- but the signature is general.
-auf :: Optic (Costar f) g s t a b -> (f a -> g b) -> f s -> g t
-auf = coerce
+--
+-- Note: The direction of the 'Iso' required for this function changed in @lens@ 4.18 to match up
+-- with the behavior of 'au'. For the old behavior use 'xplatf' or for a version that is compatible
+-- across both old and new versions of @lens@ you can just use 'coerce'!
+auf :: (Functor f, Functor g) => AnIso s t a b -> (f t -> g s) -> f b -> g a
+auf k ftgs fb = withIso k $ \sa bt -> sa <$> ftgs (bt <$> fb)
 {-# INLINE auf #-}
 
+-- | @'xplat' = 'au' . 'from'@ but with a nicer signature.
+xplat :: Optic (Costar ((->) s)) g s t a b -> ((s -> a) -> g b) -> g t
+xplat f g = xplatf f g id
+
+-- | @'xplatf' = 'auf' . 'from'@ but with a nicer signature.
+--
+-- >>> xplatf (_Unwrapping Sum) (foldMapOf both) Prelude.length ("hello","world")
+-- 10
+--
+-- @
+-- 'xplatf' :: 'Iso' s t a b -> ((r -> a) -> e -> b) -> (r -> s) -> e -> t
+-- @
+xplatf :: Optic (Costar f) g s t a b -> (f a -> g b) -> f s -> g t
+xplatf = coerce
+{-# INLINE xplat #-}
+
 -- | The opposite of working 'Control.Lens.Setter.over' a 'Setter' is working 'under' an isomorphism.
 --
 -- @
@@ -409,6 +458,57 @@ instance Swapped (,) where
 instance Swapped Either where
   swapped = iso (either Right Left) (either Right Left)
 
+instance (Swapped f, Swapped g) => Swapped (Product f g) where
+  swapped = iso f f
+    where
+      f (Pair x y) = Pair (x ^. swapped) (y ^. swapped)
+
+instance (Swapped p, Swapped q) => Swapped (Sum p q) where
+  swapped = iso f f
+    where
+      f (L2 x) = L2 (x ^. swapped)
+      f (R2 x) = R2 (x ^. swapped)
+
+instance (Swapped p) => Swapped (Flip p) where
+  swapped = iso f f
+    where
+      f (Flip p) = Flip (p ^. swapped)
+
+instance (f ~ g, Functor f, Swapped p) => Swapped (Biff p f g) where
+  swapped = iso f f
+    where
+      f (Biff p) = Biff (p ^. swapped)
+
+instance (Functor f, Swapped p) => Swapped (Tannen f p) where
+  swapped = iso f f
+    where
+      f (Tannen x) = Tannen $ fmap (^. swapped) x
+
+instance Swapped ((,,) x) where
+  swapped = iso f f
+    where
+      f (x,a,b) = (x,b,a)
+
+instance Swapped ((,,,) x y) where
+  swapped = iso f f
+    where
+      f (x,y,a,b) = (x,y,b,a)
+
+instance Swapped ((,,,,) x y z) where
+  swapped = iso f f
+    where
+      f (x,y,z,a,b) = (x,y,z,b,a)
+
+instance Swapped ((,,,,,) x y z w) where
+  swapped = iso f f
+    where
+      f (x,y,z,w,a,b) = (x,y,z,w,b,a)
+
+instance Swapped ((,,,,,,) x y z w v) where
+  swapped = iso f f
+    where
+      f (x,y,z,w,v,a,b) = (x,y,z,w,v,b,a)
+
 -- | Ad hoc conversion between \"strict\" and \"lazy\" versions of a structure,
 -- such as 'StrictT.Text' or 'StrictB.ByteString'.
 class Strict lazy strict | lazy -> strict, strict -> lazy where
